#![allow(clippy::expect_used)]
use askama::Template;
use axum::{
    Router,
    extract::{Path, State},
    http::StatusCode,
    response::{Html, IntoResponse},
    routing::get,
    serve,
};
use person::PersonID;
use stammbaum::Stammbaum;
use std::io::BufReader;
use tokio::runtime::Runtime;

mod person;
mod stammbaum;

async fn get_person(
    Path(id): Path<String>,
    State(stammbaum): State<Stammbaum>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    let id = match PersonID::try_from(id) {
        Ok(id) => id,
        Err(_) => return Err(StatusCode::INTERNAL_SERVER_ERROR),
    };
    match stammbaum.get(id) {
        None => Err(StatusCode::NOT_FOUND),
        Some(p) => Ok(Html(
            p.render().map_err(|_| StatusCode::INTERNAL_SERVER_ERROR)?,
        )),
    }
}

async fn list_stammbaum(
    State(stammbaum): State<Stammbaum>,
) -> Result<impl IntoResponse, impl IntoResponse> {
    Ok::<axum::response::Html<std::string::String>, StatusCode>(Html(stammbaum.render()))
}

pub fn run() -> Result<(), String> {
    let rt = Runtime::new().map_err(|e| e.to_string())?;
    rt.block_on(async {
        #[allow(clippy::expect_used)]
        let stammbaum: Stammbaum = serde_json::from_reader(BufReader::new(
            std::fs::File::open("test.json").expect("tbd file"),
        ))
        .expect("tbd struct");

        let app = Router::new()
            .route("/", get(list_stammbaum))
            .route("/{id}", get(get_person))
            .with_state(stammbaum);
        let addr = tokio::net::TcpListener::bind("0.0.0.0:3000")
            .await
            .expect("faild to bind port");

        serve(addr, app).await.map_err(|e| e.to_string())
    })
    .map_err(|e| e.to_string())?;

    Ok(())
}
